package edu.uiuc.ncsa.myproxy.oa4mp.loader;

import edu.uiuc.ncsa.myproxy.oa4mp.server.ServiceConstantKeys;
import edu.uiuc.ncsa.myproxy.oa4mp.server.ServiceEnvironmentImpl;
import edu.uiuc.ncsa.myproxy.oa4mp.server.servlet.ServerConfigurationLoader;
import edu.uiuc.ncsa.myproxy.oa4mp.server.util.OA4MPIdentifierProvider;
import edu.uiuc.ncsa.security.delegation.server.issuers.AccessTokenIssuer;
import edu.uiuc.ncsa.security.delegation.server.issuers.AuthorizationGrantIssuer;
import edu.uiuc.ncsa.security.delegation.server.issuers.ProtectedAssetIssuer;
import edu.uiuc.ncsa.security.delegation.server.storage.ClientApprovalStore;
import edu.uiuc.ncsa.security.delegation.server.storage.ClientStore;
import edu.uiuc.ncsa.security.delegation.storage.Client;
import edu.uiuc.ncsa.security.delegation.storage.TransactionStore;
import edu.uiuc.ncsa.security.delegation.token.TokenForge;
import edu.uiuc.ncsa.security.oauth_1_0a.OAuthConstants;
import edu.uiuc.ncsa.security.oauth_1_0a.OAuthTokenForge;
import edu.uiuc.ncsa.security.oauth_1_0a.client.OAClientProvider;
import edu.uiuc.ncsa.security.oauth_1_0a.server.AGIProvider;
import edu.uiuc.ncsa.security.oauth_1_0a.server.ATIProvider;
import edu.uiuc.ncsa.security.oauth_1_0a.server.PAIProvider;
import net.oauth.OAuth;
import org.apache.commons.configuration.tree.ConfigurationNode;

import javax.inject.Provider;
import java.util.HashMap;

import static edu.uiuc.ncsa.myproxy.oa4mp.server.util.OA4MPIdentifierProvider.*;

/**
 * This made to be over-ridden and extended with new features, hence the separate call for
 * creating the service environment. All of the elements are protected rather than private.
 * <H3>Usage</H3>
 * <UL>
 * <LI>Extend this, adding in new extensions to the XML configuration</LI>
 * <LI>Choose a servlet (such as Init) which has to be loaded first, using Tomcat's load-on-startup parameter</LI>
 * <LI>Set an instance of this as the Bootstrapper by over-riding {@link edu.uiuc.ncsa.myproxy.oa4mp.server.servlet.MyProxyDelegationServlet} loadEnvironment
 * and setting the instance directly. This side-steps Tomcat's loading issues and Java single inheritance limitations at once</LI>
 * <p/>
 * </UL>
 */
public class OA4MPConfigurationLoader<T extends ServiceEnvironmentImpl> extends ServerConfigurationLoader<T> {
    @Override
    public String getVersionNumber() {
        return "OA4MP Server version 1.0.6";
    }

    @Override
    public Provider<AuthorizationGrantIssuer> getAGIProvider() {
        return new AGIProvider(getTokenForgeProvider().get(), getServiceAddress());
    }

    @Override
    public Provider<ClientStore> getClientStoreProvider() {
        return getCSP();
    }

    @Override
    public Provider<ClientApprovalStore> getClientApprovalStoreProvider() {
        return getCASP();
    }

    @Override
    public Provider<TransactionStore> getTransactionStoreProvider() {
        return getTransactionProvider();
    }

    @Override
    public Provider<TokenForge> getTokenForgeProvider() {
        return new OAuthForgeProvider();
    }

    @Override
    public Provider<AccessTokenIssuer> getATIProvider() {
        return new ATIProvider(getTokenForgeProvider().get(), getServiceAddress());
    }

    @Override
    public Provider<ProtectedAssetIssuer> getPAIProvider() {
        return new PAIProvider(getTokenForgeProvider().get(), getServiceAddress());
    }

    public OA4MPConfigurationLoader(ConfigurationNode node) {
        super(node);
   }

    @Override
    public Provider<? extends Client> getClientProvider() {
        // Do *not* use timestamps when making these, since this would be redundant.
        return new OAClientProvider(new OA4MPIdentifierProvider(SCHEME, SCHEME_SPECIFIC_PART, CLIENT_ID, false));
    }

    protected static class OAuthForgeProvider implements Provider<TokenForge>{
        public TokenForge get(){
            return new OAuthTokenForge("myproxy:oa4mp,2012:");
        }
    }

    HashMap<String,String> constants;

    @Override
    public HashMap<String, String> getConstants() {
        if(constants == null){
            constants = new HashMap<String, String>();
            // OAuth 1.0a callback constant. This is used to as a key for http request parameters
            constants.put(ServiceConstantKeys.CALLBACK_URI, OAuthConstants.OAUTH_CALLBACK);
            constants.put(ServiceConstantKeys.TOKEN, OAuth.OAUTH_TOKEN);
            constants.put(ServiceConstantKeys.FORM_ENCODING, OAuthConstants.FORM_ENCODING);
        }
        return constants;
    }
}